#include <stdarg.h>
#include <string.h>
#include <stdint.h>

#define LCD_CLR_DISP 0x1

#define LCD_HOME 0x2

#define LCD_ENTRY_SET 0x4
#define LCD_ENTRY_SET_CURS_RIGHT 0x2
#define LCD_ENTRY_SET_SHIFT 0x1

#define LCD_DISP_ONOFF 0x8
#define LCD_DISP_ONOFF_DISP 0x4
#define LCD_DISP_ONOFF_CURS 0x2
#define LCD_DISP_ONOFF_BLINK 0x1

#define LCD_FUNC_SET 0x20
#define LCD_FUNC_SET_8BIT 0x10
#define LCD_FUNC_SET_2LINE 0x8
#define LCD_FUNC_SET_5X11 0x4

#ifndef LINT
  __sfr __at 0x10 lcd_control;
  __sfr __at 0x11 lcd_data_wr;
#endif

#ifdef LINT
  uint8_t lcd_control;
  uint8_t lcd_data_wr;
#endif

void lcd_init() {
  // __asm  // Keep this here, makes it work. 
  //   nop
  //   nop
  //   nop
  //   nop
  // __endasm ;
  lcd_control=LCD_FUNC_SET|LCD_FUNC_SET_8BIT|LCD_FUNC_SET_2LINE;
  lcd_control=LCD_DISP_ONOFF|LCD_DISP_ONOFF_DISP|LCD_DISP_ONOFF_CURS;
  lcd_control=LCD_CLR_DISP;
  // lcd_control=LCD_ENTRY_SET|LCD_ENTRY_SET_CURS_RIGHT;
  // lcd_control=LCD_HOME;
}


static void write_string(const char* s) {
  for (int i=0;s[i]!='\0';i++) {
    lcd_data_wr=s[i];
  }
}

void lcd_printf(const char* format,...) {
  va_list arg;
  va_start(arg,format);
  for(;*format!='\0';format++) {
    if(*format!='%') {
      lcd_data_wr=*format;
      continue;
    }
    format++;
    switch(*format) {
      case 'c': {
        char i=(char)va_arg(arg,int);
        lcd_data_wr=i;
        break;
      }
      case 'd': {
        int i=va_arg(arg,int); 		//Fetch Decimal/Integer argument
        if(i<0) {
          i=-i;
          lcd_data_wr='-';
        }
        char str[11];
        int_to_ascii(i,str);
        write_string(str);
        break;
      }
      // case 'o': {
      //   int i=va_arg(arg,unsigned int); //Fetch Octal representation
      // 	puts(convert(i,8));
      // 	break;
      // }
      case 's': {
        char* s=va_arg(arg,char*);
        write_string(s);
        break;
      }
      case 'x': {
        uint32_t i=va_arg(arg,uint32_t);
        char str[11];
        str[0]='\0';
        hex_to_ascii(i,str);
        write_string(str);
        break;
      }
    }
  }
  va_end(arg);
}
