#include "lcd.h"
#include <stdint.h>
#define BITMAP_BANK 1
#define NUM_BITS 65536
uint8_t vpmem_num_banks;
#ifndef LINT
__sfr __at 0x0 bank0;
#endif

#ifdef LINT
  uint8_t bank0;
#endif

volatile uint8_t* bitmap_bank_ptr;

static uint8_t get_bmap_bit(uint16_t bit) {
  uint8_t bit_no=bit%8;
  bank0=BITMAP_BANK;
  //bank0=bitmap_banks[bit>>17];
  uint8_t byte=bitmap_bank_ptr[bit/8];
  return (byte&(1<<bit_no))>>bit_no;
}

static void set_bmap_bit(uint16_t bit) {
  uint16_t byte_no=bit>>3;
  bank0=BITMAP_BANK;
  uint8_t byte=bitmap_bank_ptr[byte_no];
  byte=byte|(1<<(bit&0x7));
  bitmap_bank_ptr[byte_no]=byte;
}

static void clear_bmap_bit(uint16_t bit) {
  uint16_t byte_no=bit>>3;
  bank0=BITMAP_BANK;
  uint8_t byte=bitmap_bank_ptr[byte_no];
  byte=byte&(~(1<<(bit&0x7)));
  bitmap_bank_ptr[byte_no]=byte;
}

void vpmem_free(uint16_t start, uint16_t length) {
  for (uint16_t i=0;i<length;i++) {
    clear_bmap_bit(start+i);
  }
}

uint16_t vpmem_alloc(uint16_t num_pages) {
  uint16_t bmap_index=0xFFFF;
  uint16_t remaining_blks=num_pages;
  for(uint32_t i=0;i<NUM_BITS;i++) {
    uint8_t got_0=0;
    uint16_t old_j=0;
    for (uint16_t j=i*8;;j++) {
      uint8_t bit=get_bmap_bit(j);
      if (got_0) {
        if (bit) {
          if (remaining_blks==0) {
              bmap_index=old_j;
              break;
          } else {
            i+=j/8;
            i--;
            break;
          }
        } else {
          remaining_blks--;
        }
      } else {
        if (!bit) {
          got_0=1;
          old_j=j;
          remaining_blks--;
        }
      }
      if (remaining_blks==0) {
        bmap_index=old_j;
        break;
      }
    }
    if (remaining_blks==0) {
      break;
    }
  }
  if (remaining_blks!=0) {
    return 0x0;
  }
  for (uint16_t i=0;i<num_pages;i++) {
    set_bmap_bit(bmap_index+i);
  }
  return bmap_index;
}

void vpmem_init()  {
  bank0=BITMAP_BANK;
  bitmap_bank_ptr=0x0;
  for (uint8_t i=0;i<4;i++) {
    bitmap_bank_ptr[i]=0x00;
  }
  for (uint8_t i=4;i<255;i++) {
    bitmap_bank_ptr[i]=0xFF;
  }
  set_bmap_bit(0);
  set_bmap_bit(1);
  vpmem_num_banks=32;
}
